/*#############################
 *	Author: Mark Walker
 *	Class: AABB.java
 *		The axis-aligned bounding box class
 *############################*/
public class AABB extends PointMass			//Axis-Aligned Bounding Box class
{
	private Vector2D w, h;	//half-width and half-height vectors
	private int CLASS = 2;
	
	public AABB (double mass, int width, int height, Vector2D l)	//l is location vector for the CENTER
	{																//of the AABB
		super(mass,l);
		w = new Vector2D(width/2,0);
		h = new Vector2D(0,height/2);
		I = .3333*4*(getWidth()*getWidth() + getHeight()*getHeight());
	}
	
	public boolean collideCircle(BoundingCircle b, double x, double y)	//x and y b's coords local to this aabb
	{
		if (!quickCircleTest(b.getRadius(),x,y))
			return false;
		if (PhysicsEngine.pointLineSqDist((double)getWidth(),(double)getHeight(),x,y) < b.getRadius()*b.getRadius())
			return true;
		if (PhysicsEngine.pointLineSqDist((double)getWidth(),(double)-getHeight(),x,y) < b.getRadius()*b.getRadius())
			return true;
		if (PhysicsEngine.pointLineSqDist((double)-getWidth(),(double)getHeight(),x,y) < b.getRadius()*b.getRadius())
			return true;
		if (PhysicsEngine.pointLineSqDist((double)-getWidth(),(double)-getHeight(),x,y) < b.getRadius()*b.getRadius())
			return true;
		return true;
	}
	
	private boolean quickCircleTest(double r, double x, double y)	//quick 'n easy square distance test
	{																		//NOT a final test
		//find squared distance from closest point to the circle center
		//and see if it is less than the squared radius.
		if (Math.sqrt(getHeight()*getHeight() + getWidth()*getWidth()) > Math.sqrt(x*x + y*y) - r)
			return true;
		return false;
	}
	
	public boolean collideOBB(OBB b, double x, double y)
	{
		return false;
	}
	public boolean collideAABB(AABB b, double x, double y)
	{
		//treat b as a bindingcircle with radius of the distance from center to a corner (shortcut check?)
		if (!quickCircleTest(Math.sqrt(b.getWidth()*b.getWidth() + b.getHeight()*b.getHeight()),x,y))
			return false;
		
		//simply check to see if they overlap:
		//if b's left or right edge is between or envelopes this object's vertical edges
		/*if (((int)x+b.getWidth() >= -getWidth() && (int)x+b.getWidth() <= getWidth()) ||
			((int)x-b.getWidth() >= -getWidth() && (int)x-b.getWidth() <= getWidth()) ||
			((int)x-b.getWidth() <= -getWidth() && (int)x+b.getWidth() >= getWidth()))
			//if b's top or bottom edge is between or envelopes this object's horizontal edges
			if (((int)y+b.getHeight() >= -getHeight() && (int)y+b.getHeight() <= getHeight()) || 
			((int)y-b.getHeight() >= -getHeight() && (int)y-b.getHeight() <= getHeight()) ||
			((int)y-b.getHeight() <= -getHeight() && (int)y+b.getHeight() >= getHeight()))
				return true;*/
		if (PhysicsEngine2.AASideOverlap(x-b.getWidth(),x+b.getWidth(),-getWidth(),getWidth()) && 
			PhysicsEngine2.AASideOverlap(y-b.getHeight(),y+b.getHeight(),-getHeight(),getHeight()))
			return true;
		
		return false;
	}
	
	public int getWidth()		//returns halfwidth variable
	{
		return (int)w.getI();
	}
	
	public int getHeight()		//returns halfheight variable
	{
		return (int)h.getJ();
	}
	
	public Vector2D getWidthVector()
	{
		return w;
	}
	
	public Vector2D getHeightVector()
	{
		return h;
	}
	
	public int CLASS()
	{
		return 2;
	}
}