import java.util.*;

/*#############################
 *	Author: Mark Walker
 *	Class: BlockGame.java
 *		A simple game that lets you play with AABBs
 *############################*/

public class BlockGame implements GameEngine
{
	private PhysicsEngine2 phys;
	
	private int HEIGHT = 70, WIDTH = 70, MASS = 1000, GRAVITY = 7000, V_EXPL = 3000;
	public boolean bStatic;
	
	public BlockGame()
	{
		phys = new PhysicsEngine2();
		
		bStatic = false;
		
		AABB b = new AABB(1,900,200,new Vector2D(400,800));		//border boxes
		b.setStatic(true);
		AABB c = new AABB(1,900,200,new Vector2D(400,0));
		c.setStatic(true);
		AABB d = new AABB(1,200,900,new Vector2D(0,400));
		d.setStatic(true);
		AABB e = new AABB(1,200,900,new Vector2D(800,400));
		e.setStatic(true);
		getEnv().addObject(b);
		getEnv().addObject(c);
		getEnv().addObject(d);
		getEnv().addObject(e);
	}
	
	public void changeStaticMode()
	{
		System.out.println("Static mode changed.");
		bStatic = !bStatic;
	}
	
	public void addObject(int x, int y)
	{
		System.out.println("Adding box at location (" + x + ", " + y + ")");
		AABB box = new AABB(MASS,WIDTH,HEIGHT,new Vector2D(x,y));
		getEnv().addObject(box);
		box.setAccel(new Vector2D(0,GRAVITY));	
		if (bStatic)
			box.setStatic(true);	
	}
	
	public void explosion(int x, int y)
	{
		System.out.println("EXPLOSION!");
		ArrayList l = getEnv().getAllObjects();
		AABB box;
		for (int i = 0; i < l.size(); i++)
		{
			box = (AABB)l.get(i);
			if (!box.bStatic() || box.bSleeping())
			{
				box.setVelocity(new Vector2D(V_EXPL*-Math.sin(getEnv().getLoc(box).getI()/2),
					V_EXPL*-Math.cos(getEnv().getLoc(box).getJ()/2)));
				box.wake();
			}
		}
	}
	
	public void act()
	{
		phys.act();
	}
	
	public PhysicsEnv getEnv()
	{
		return phys.getEnv();
	}
}