/*#############################
 *	Author: Mark Walker
 *	Class: BoundingCircle.java
 *		This is the class for bounding circles
 *############################*/
import java.util.*;

public class BoundingCircle extends PointMass
{
	private double r;
	private int CLASS = 1;
	
	public BoundingCircle(double mass, double rad, Vector2D l)
	{
		super(mass,l);
		r = rad;
		//cylindrical moment of inertia
		I = .5 * mass * r * r;
	}
	
	public boolean collideCircle(BoundingCircle b, double x, double y)		//x and y in local coordinates
	{
		//System.out.println("Circle in local coordinates: " + x + ", " + y);
		//check to see if the squared distance to b is less than the square of the sum of the radii
		double d2 = r + b.getRadius();
		if (x*x + y*y < d2*d2)
			return true;
		return false;
	}
	
	public boolean collideOBB(OBB b, double x, double y)
	{
		return false;
	}
	public boolean collideAABB(Vector2D topLeft,Vector2D topRight,Vector2D bottomLeft,
		Vector2D bottomRight,Vector2D loc)
	{		
		if (PhysicsEngine2.AASideOverlap(loc.getI(),loc.getI(),topLeft.getI(),topRight.getI()) &&
			PhysicsEngine2.AASideOverlap(loc.getJ(),loc.getJ(),topLeft.getJ(),bottomLeft.getJ()))
			return true;
	
		if (PhysicsEngine2.pointLineSqDist(topLeft,topRight,loc) < r*r)
			return true;
		if (PhysicsEngine2.pointLineSqDist(topLeft,bottomLeft,loc) < r*r)
			return true;
		if (PhysicsEngine2.pointLineSqDist(topRight,bottomRight,loc) < r*r)
			return true;
		if (PhysicsEngine2.pointLineSqDist(bottomLeft,bottomRight,loc) < r*r)
			return true;
		return false;
	}
	
	public boolean collideAABB(AABB b, Vector2D locb, Vector2D loca)
	{
		Vector2D topLeft = locb.subtract(b.getWidthVector()).subtract(b.getHeightVector());
		Vector2D topRight = locb.add(b.getWidthVector()).subtract(b.getHeightVector());
		Vector2D bottomLeft = locb.subtract(b.getWidthVector()).add(b.getHeightVector());
		Vector2D bottomRight = locb.add(b.getWidthVector()).add(b.getHeightVector());
		
		return collideAABB(topLeft,topRight,bottomLeft,bottomRight,loca);
	}
	
	public Vector2D getRadiusVector()
	{	//returns vector2d on +x axis with length r
		return new Vector2D(r,0);
	}
	
	public double getRadius()
	{
		return r;
	}
	
	public int CLASS()
	{
		return 1;
	}
}