/*#############################
 *	Author: Mark Walker
 *	Class: CircleGame.java
 *	A simple game engine that lets you play with bounding circles	
 *############################*/

import java.util.*;

public class CircleGame implements GameEngine
{
	private PhysicsEngine2 phys;
	
	private int RADIUS = 35, MASS = 1000, GRAVITY = 10000, V_EXPL = 1000;
	public boolean bStatic;
	
	public CircleGame()
	{
		phys = new PhysicsEngine2();
		
		bStatic = false;
		
		AABB b = new AABB(1,900,200,new Vector2D(400,800));		//border boxes
		b.setStatic(true);
		AABB c = new AABB(1,900,200,new Vector2D(400,0));
		c.setStatic(true);
		AABB d = new AABB(1,200,900,new Vector2D(0,400));
		d.setStatic(true);
		AABB e = new AABB(1,200,900,new Vector2D(800,400));
		e.setStatic(true);
		getEnv().addObject(b);
		getEnv().addObject(c);
		getEnv().addObject(d);
		getEnv().addObject(e);
	}
	
	public void changeStaticMode()
	{
		System.out.println("Static mode changed.");
		bStatic = !bStatic;
	}
	
	public void addObject(int x, int y)
	{
		System.out.println("Adding circle at location (" + x + ", " + y + ")");
		BoundingCircle circle = new BoundingCircle(MASS,RADIUS,new Vector2D(x,y));
		getEnv().addObject(circle);
		circle.setAccel(new Vector2D(0,GRAVITY));	
		if (bStatic)
			circle.setStatic(true);	
	}
	
	public void explosion(int x, int y)
	{
		System.out.println("EXPLOSION!");
		ArrayList l = getEnv().getAllObjects();
		BoundingCircle circle;
		for (int i = 4; i < l.size(); i++)
		{
			circle = (BoundingCircle)l.get(i);
			if (!circle.bStatic() || circle.bSleeping())
			{
				circle.setVelocity(new Vector2D(V_EXPL*-Math.sin(getEnv().getLoc(circle).getI()/2),
					-V_EXPL));
				circle.wake();
			}
		}
	}
	
	public void act()
	{
		phys.act();
	}
	
	public PhysicsEnv getEnv()
	{
		return phys.getEnv();
	}
}