/*#############################
 *	Author: Mark Walker
 *	Class: PhysicsEngine2.java
 *		This is the main physics engine that keeps track of the environment, time, handles collisions, etc
 *############################*/

import java.util.*;

public class PhysicsEngine2
{
	private PhysicsEnv env;
	private long t;
	private double dt;
	
	//E is the "coefficient of restitution" (Palmer, 145), a number between 0 and 1 that controls
	//	dampening of collisions, where 1 means elastic, and 0 is completely inelastic (stuck together)
	
	//CE is the coefficient of restitution for circle/circle collisions
	
	//CC_N is the multiplier for the normal force of one circle on another (should not be more than 1)
	
	//STEP_SIZE is the amount an object is traced forward/back each step during collision detection
	
	//MAX_ITR is the maximum number of those steps that will be taken
	
	//COLL_ITR is the maximum number of time all objects will be tested for collision and response
	
	//WAKE_KE is the minimum amount of 'KE' that an object must have to wake up another
	
	//SLEEP_KE is the minimum amount of 'KE' that an object can have before falling asleep
	
	private final double E = .3, E_X = .8, CE = .8, CC_N = 1, STEP_SIZE = 1, MAX_ITR = 20, COLL_ITR = 3,
		 WAKE_KE =8000, SLEEP_KE = 5000;
	private Vector2D lastV;
	
	public PhysicsEngine2()
	{
		env = new PhysicsEnv();
		t = System.currentTimeMillis();
		dt = 0;
		lastV = new Vector2D(0,0);
	}
	
	public void act()
	{
		//find delta time
		/*long t2 = System.currentTimeMillis();
		dt = (double)(t2 - t)/1000.0;
		if (dt == 0.0)			//don't run all this if no time has gone by
			return;
		t = t2;*/
		dt = .01;				//Comment out this line to do 'real' time simulation
		PointMass p, b;			//all p's being tested against b's
		boolean v, bCollision = true;				//true if there was a collision between a and b
		Vector2D w;				//change in position for p
		int n = 0;		
		
		unflagAll();			//unflag all objects
		
		while (bCollision && n < COLL_ITR)
		{
			bCollision = false;
			for (int i = 0; i < env.getAllObjects().size(); i++)	//iterate through all objects to check for collisions
			{														//...against all other objects
				p = (PointMass)(env.getAllObjects().get(i));
				if (p == null)
					break;
				//p.unflag();
				
				/*if (p.bSleeping())		//make sure there is something holding up sleeping objects
				{
					//v = sleepCollision(p);
					//if (!v)
						//p.wake();
				}*/
				
				for (int j = 0; j < env.getAllObjects().size() && p.bCollidable() 
					&& !p.bStatic(); j++)
				{
					if (j != i)			//do not collide with self
					{
						b = (PointMass)(env.getAllObjects().get(j));		//object being tested against
						if (b != null && b.bCollidable())
						{
							v = collision(p,b);			//find exact collision
							if (v)
							{
								//System.out.println("velocity = " + p.getVelocity() + "; acceleration = " + p.getAcceleration());
								
								//System.out.println("getKE() = " + p.getKE()/p.getMass() + "; actualKE() = " + p.actualKE(dt)/p.getMass());
								if (p.actualKE(dt)/p.getMass() >= WAKE_KE && b.bSleeping())
								{
									//System.out.println("Act::74: b.wake()");
									b.wake();
								}
								//System.out.println("lastV = " + lastV);
								if (p.actualKE(dt)/p.getMass() < SLEEP_KE && b.bStatic()) //&& p.getAcceleration().dot(lastV) < 0)
								{
									//System.out.println("Act::68: p.sleep()");
									p.sleep();
								}
								bCollision = true;
								if (!b.bStationary())	//if 'b' isn't stationary, flag p
									p.flag();
								//System.out.println("Collision!");
							}
						}
					}
				}
			}
			Constraint c;
			for (int i = 0; i < env.getAllConstraints().size(); i++)
			{
				c = (Constraint)env.getAllConstraints().get(i);
				c.updateConstraint(dt,env);
			}
			n++;
		}
		for (int i = 0; i < env.getAllObjects().size(); i++)
		{
			if (!((PointMass)env.getAllObjects().get(i)).bSleeping())
			{
				w = ((PointMass)env.getAllObjects().get(i)).act(dt);		//act and return change in position
				env.moveObject((PointMass)env.getAllObjects().get(i),w);		//update environment
				//((PointMass)env.getAllObjects().get(i)).unflag();
			}
		}
	}
	
	private void unflagAll()
	{
		ArrayList l = env.getAllObjects();
		for (int i = 0; i < l.size(); i++)
		{
			((PointMass)l.get(i)).unflag();
		}
	}
	
	private boolean sleepCollision(PointMass p)	//test used to make sure a sleeping object
	{											//is not overlapping
		p.wake();
		PointMass b;
		boolean bCollision = false;
		int s = 0;
		int itr = (int)MAX_ITR;
		for (int u = 0; u < env.getAllObjects().size(); u++)
		{
			b = (PointMass)(env.getAllObjects().get(u));
			if (b.bStatic() && b != p)
			{
				s = 0;
				//System.out.println("Sleep collision testing...");
				while (collision(p,b) && s < itr)
				{
					bCollision = true;
					//System.out.println("Correcting sleeper's position...");
					s++;
				}
				if (s == MAX_ITR)
				{
					//sleepCollision(b);
					//u = 0;
					//itr *= 2;
				}
			}
		}
		//if (!bCollision)
			p.sleep();
			
		return bCollision;
	}
	
	private boolean collision(PointMass a, PointMass b)
	{
		//each type of object given a defined CLASS value (see PointMass header)
		//System.out.println("PhysicsEngine::60: identifying collision type...");
		if (a.CLASS() == 1)
		{
			if (b.CLASS() == 1)
				return collisionCC((BoundingCircle)a,(BoundingCircle)b);
			if (b.CLASS() == 2)
				return collisionCA((BoundingCircle)a,(AABB)b);
			if (b.CLASS() == 3)
				return collisionCO(a,b);
		}
		if (a.CLASS() == 2)
		{
			if (b.CLASS() == 1)
			{
				return collisionCA((BoundingCircle)b,(AABB)a);
			}
			if (b.CLASS() == 2)
				return collisionAA((AABB)a,(AABB)b);
			if (b.CLASS() == 3)
				return collisionAO(a,b);
		}
		if (a.CLASS() == 3)
		{
			if (b.CLASS() == 1)
			{
				return collisionCO(b,a);
			}
			if (b.CLASS() == 2)
			{
				return collisionAO(b,a);
			}
			if (b.CLASS() == 3)
				return collisionOO(a,b);
		}
		System.out.println("PhysicsEngine::185: failed to detect collision type.");
		return false;
	}
	
	public boolean collisionCC(BoundingCircle a, BoundingCircle b)
	{
		Vector2D av = a.getVelocity();
		Vector2D bv = b.getVelocity();
		if (!a.collideCircle(b,(b.getLoc().getI() - env.nextLoc(a,dt).getI()), (b.getLoc().getJ() - env.nextLoc(a,dt).getJ())))
			return false;
		
		int sx,sy, u = 0;
		if (env.getLoc(a).getI() < env.getLoc(b).getI())
			sx = -1;
		else
			sx = 1;
			
		if (env.getLoc(a).getJ() < env.getLoc(b).getJ())
			sy = -1;
		else
			sy = 1;
		
		Vector2D w = new Vector2D(sx*STEP_SIZE,sy*STEP_SIZE);
		w = new Vector2D( (b.getLoc().getI() - env.nextLoc(a,dt).getI()), (b.getLoc().getJ() - env.nextLoc(a,dt).getJ()) );
		w = w.scalarMult(-STEP_SIZE/w.magnitude());
		while (u < MAX_ITR &&
				a.collideCircle(b,env.getLoc(b).getI()-env.getLoc(a).getI(),env.getLoc(b).getJ()-env.getLoc(a).getJ()))
		{
			env.moveObject(a,w);
			u++;
		}
		
		double by = b.getVelocity().getJ();
		double bx = b.getVelocity().getI();
		
		//normal force
		Vector2D n = new Vector2D(env.getLoc(b).getI()-env.getLoc(a).getI(),env.getLoc(b).getJ()-env.getLoc(a).getJ());
		n = n.scalarDiv(n.magnitude());
		lastV = n.scalarMult(-1);
		n = n.scalarMult(b.getAcceleration().magnitude()).scalarMult(dt*Math.sin(b.getAcceleration().angle(n)));
		
		if (!b.bStatic()) //|| b.bSleeping())
		{
			b.getVelocity().setJ( (((b.getVelocity().getJ()*(b.getMass()-(CE*a.getMass())))/(a.getMass()+b.getMass())) +
				((a.getVelocity().getJ()*(a.getMass()*(1+CE)))/(a.getMass()+b.getMass()))));
			
			b.getVelocity().setI( (((b.getVelocity().getI()*(b.getMass()-(CE*a.getMass())))/(a.getMass()+b.getMass())) +
				((a.getVelocity().getI()*(a.getMass()*(1+CE)))/(a.getMass()+b.getMass()))));
			
			a.getVelocity().setJ( (((a.getVelocity().getJ()*(a.getMass()-(CE*b.getMass())))/(a.getMass()+b.getMass())) +
				((by*(b.getMass()*(1+CE)))/(a.getMass()+b.getMass()))));
			
			a.getVelocity().setI( (((a.getVelocity().getI()*(a.getMass()-(CE*b.getMass())))/(a.getMass()+b.getMass())) +
				((bx*(b.getMass()*(1+CE)))/(a.getMass()+b.getMass()))));
				
			//b.setVelocity(b.getVelocity().setAngle(w.scalarMult(-1).getAngle()));
			//a.setVelocity(a.getVelocity().setAngle(w.scalarMult(1).getAngle()));
			
			//normal force of a on b
			b.setVelocity(b.getVelocity().add(n).scalarMult(CC_N));
		} else
		{
			a.setVelocity(a.getVelocity().setAngle(w.scalarMult(1).getAngle()));
		}
		
		//normal force of b on a
		a.setVelocity(a.getVelocity().add(n.scalarMult(-CC_N)));
		
		return true;
	}
	
	//Collision between Circle and AABB
	public boolean collisionCA(BoundingCircle a, AABB b)
	{
		//bound AABB into a circle and do circle/circle test FIRST (faster) using no square roots
		//instead of taking square root of the sum of the squares of the dimensions, just add width+height to approximate
		if (!a.collideCircle(new BoundingCircle(b.getMass(),
			b.getWidth()+b.getHeight(),env.nextLoc(b,dt)),env.nextLoc(b,dt).getI() - env.nextLoc(a,dt).getI(),
			env.nextLoc(b,dt).getJ() - env.nextLoc(a,dt).getJ()))
			return false;
		
		//more rigorous collision test		
		Vector2D topLeft = env.nextLoc(b,dt).subtract(b.getWidthVector()).subtract(b.getHeightVector());
		Vector2D topRight = env.nextLoc(b,dt).add(b.getWidthVector()).subtract(b.getHeightVector());
		Vector2D bottomLeft = env.nextLoc(b,dt).subtract(b.getWidthVector()).add(b.getHeightVector());
		Vector2D bottomRight = env.nextLoc(b,dt).add(b.getWidthVector()).add(b.getHeightVector());
		Vector2D p1 = env.nextLoc(a,dt), p2 = env.nextLoc(b,dt);
		
		if (!a.collideAABB(topLeft,topRight,bottomLeft,bottomRight,env.nextLoc(a,dt)))
			return false;
			
		
		//System.out.println("Circle/AABB collision.");
		double xf, yf, x0, y0, bw, bh;
		if (!a.bStatic() || (a.getVelocity().getI() != 0 && a.getVelocity().getJ() != 0))
		{	
			//if a is not stationary
			xf = env.nextLoc(a,dt).getI() - env.nextLoc(b,dt).getI();	//old and new positions
			yf = env.nextLoc(a,dt).getJ() - env.nextLoc(b,dt).getJ();
			x0 = xf - (a.getVelocity().getI()+(a.getAcceleration().getI()*dt))*dt;
			y0 = yf - (a.getVelocity().getJ()+(a.getAcceleration().getJ()*dt))*dt;
			bw = b.getWidth();
			bh = b.getHeight();
		}
		else	//if a is stationary, move a to b with negative Vb
		{
			xf = env.nextLoc(a,dt).getI() - env.nextLoc(b,dt).getI();	//old and new positions
			yf = env.nextLoc(a,dt).getJ() - env.nextLoc(b,dt).getJ();
			x0 = xf + (b.getVelocity().getI()+(b.getAcceleration().getI()*dt))*dt;
			y0 = yf + (b.getVelocity().getJ()+(b.getAcceleration().getJ()*dt))*dt;
			bw = b.getWidth();
			bh = b.getHeight();
		}
		//System.out.println("xf = " + xf + "; yf = " + yf + "; x0 = " + x0 + "; y0 = " + y0);
		Vector2D v = CAFromSide(xf,yf,x0,y0,bw,bh);
		
		int sx,sy, u = 0;
		if (env.getLoc(a).getI() < env.getLoc(b).getI())
			sx = -1;
		else
			sx = 1;
			
		if (env.getLoc(a).getJ() < env.getLoc(b).getJ())
			sy = -1;
		else
			sy = 1;
		
		lastV = new Vector2D(sx,sy);
		
		double bx = b.getVelocity().getI();
		double by = b.getVelocity().getJ();
		double ay = a.getVelocity().getJ();
		double ax = a.getVelocity().getI();
		
		
		if (v.getI() != 0 && v.getJ() == 0)		//side collision
		{
			Vector2D w = new Vector2D(sx*STEP_SIZE,0);
			//System.out.println("w = " + w + "; u = " + u);
			if (!a.bStatic())
				while (u < MAX_ITR && a.collideAABB(b,env.nextLoc(b,dt),env.nextLoc(a,dt)))
				{
					env.moveObject(a,w);
					u++;
				}
			else
				while (u < MAX_ITR && a.collideAABB(b,env.nextLoc(b,dt),env.nextLoc(a,dt)))
				{
					env.moveObject(b,w.scalarMult(-1));
					u++;
				}
			if (u == MAX_ITR)
			{
				//System.out.println("Unable to resolve circle location, moving back.");
				env.moveObject(a,w.scalarMult(-u));
			}
			//how much a will be moved by (or b moved back by)
			//double dx = env.nextLoc(b,dt).getI()+(b.getWidth()*s)-env.nextLoc(a,dt).getI()+(a.getRadius()*s);
			//System.out.println("Side collision (circle/AABB)...");
			
			if (b.bStationary())
			{
				a.getVelocity().setJ(E*a.getVelocity().getJ());
				a.getVelocity().setI(-E*a.getVelocity().getI());		
			}  else if (!a.bStationary())
			{
				a.getVelocity().setI( (((ax*(a.getMass()-(E*b.getMass())))/(a.getMass()+b.getMass())) +
				((bx*(b.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
				a.getVelocity().setJ( E*a.getVelocity().getJ() );
			}
			
			if (a.bStationary())
			{
				b.getVelocity().setJ(E*b.getVelocity().getJ());
				b.getVelocity().setI(-E*b.getVelocity().getI());
			} else if (!b.bStationary())
			{
				b.getVelocity().setI( (((bx*(b.getMass()-(E*a.getMass())))/(a.getMass()+b.getMass())) +
				((ax*(a.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
				b.getVelocity().setJ( E*b.getVelocity().getJ() );
			}
			
			if (!b.bStationary())
			{
				//normal force of a on b
				Vector2D n = new Vector2D(-b.getAcceleration().getI()*dt,0);
				b.setVelocity(b.getVelocity().add(n.scalarMult(a.getMass()/b.getMass())));
			}
			
			if (!a.bStationary())
			{
				//normal force of b on a
				Vector2D n = new Vector2D(-a.getAcceleration().getI()*dt,0);
				a.setVelocity(a.getVelocity().add(n.scalarMult(b.getMass()/a.getMass())));
			}
			
			/*//normal force
			Vector2D n = new Vector2D(sx,0);
			double phi = a.getAcceleration().angle(n);
			n = n.scalarMult(-a.getAcceleration().magnitude()*Math.cos(phi));
			
			if (!a.bStatic())
				a.setVelocity(a.getVelocity().add(n.scalarMult(b.getMass()/a.getMass())));	
			
			//System.out.println("N1 = " + n + "; accel = " + a.getAcceleration().magnitude() + "; phi = " +phi);
			
			 n = new Vector2D(-sx,0);
			phi = b.getAcceleration().angle(n);
			n = n.scalarMult(-b.getAcceleration().magnitude()*Math.cos(phi));
			
			if (!b.bStatic())
			{
				b.setVelocity(b.getVelocity().add(n.scalarMult(a.getMass()/b.getMass())));
			}*/
			
			return true;
		}
		
		if (v.getJ() != 0 && v.getI() == 0)		//top/bottom collision
		{
			Vector2D w = new Vector2D(0,sy*STEP_SIZE);
			//System.out.println("w = " + w + "; u = " + u);
			if (!a.bStatic())
				while (u < MAX_ITR && a.collideAABB(b,env.getLoc(b),env.getLoc(a)))
				{
					//System.out.println("Moving 'a' by w = " + w);
					env.moveObject(a,w);
					u++;
				}
			else
				while (u < MAX_ITR && a.collideAABB(b,env.getLoc(b),env.getLoc(a)))
				{
					//System.out.println("Moving 'b' by w = " + w);
					env.moveObject(b,w.scalarMult(-1));
					u++;
				}
			if (u == MAX_ITR)
			{
				//System.out.println("Unable to resolve circle location, moving back.");
				env.moveObject(a,w.scalarMult(-u));
			}
			
			//System.out.println("Top/bottom collision (circle/AABB)...");
			
			//how much a will be moved by (or b moved back by)
			//double dy = env.nextLoc(b,dt).getJ()+(b.getHeight()*s)-env.nextLoc(a,dt).getJ()+(a.getRadius()*s);
			if (b.bStationary())
			{
				a.getVelocity().setJ( -E*a.getVelocity().getJ() );
				a.getVelocity().setI( E*a.getVelocity().getI() );
			} else if (!a.bStationary())
			{
				a.getVelocity().setJ( (((ay*(a.getMass()-(E*b.getMass())))/(a.getMass()+b.getMass())) +
				((by*(b.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
				a.getVelocity().setI( E*a.getVelocity().getI() );
			}
			
			if (a.bStationary())
			{
				b.getVelocity().setJ( -E*b.getVelocity().getJ() );
				b.getVelocity().setI( E*b.getVelocity().getI() );
			} else if (!b.bStationary())
			{
				b.getVelocity().setJ( (((by*(b.getMass()-(E*a.getMass())))/(a.getMass()+b.getMass())) +
				((ay*(a.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
				b.getVelocity().setI( E*b.getVelocity().getI() );
			}
			
			if (!b.bStationary() && sy > 1)
			{
				//normal force of a on b
				Vector2D n = new Vector2D(0,-b.getAcceleration().getJ()*dt);
				b.setVelocity(b.getVelocity().add(n));
			}
			
			if (!a.bStationary() && sy < 1)
			{
				//System.out.println("Va before = " +a.getVelocity());
				//normal force of b on a
				Vector2D n = new Vector2D(0,-a.getAcceleration().getJ()*dt);
				a.setVelocity(a.getVelocity().add(n));
				//System.out.println("Va after = " +a.getVelocity());
			}
			
			/*//normal force
			Vector2D n = new Vector2D(0,sy);
			double phi = a.getAcceleration().angle(n);
			n = n.scalarMult(-a.getAcceleration().magnitude()*Math.cos(phi)*dt);
			if(!a.bStatic())
				a.setVelocity(a.getVelocity().add(n.scalarMult(b.getMass()/a.getMass())));
			
			//System.out.println("N1 = " + n + "; accel = " + a.getAcceleration().magnitude() + "; phi = " +phi);	
			
			n = new Vector2D(0,-sy);
			 phi = b.getAcceleration().angle(n);
			n = n.scalarMult(-b.getAcceleration().magnitude()*Math.cos(phi)*dt);
			
			if (!b.bStatic())
			{
				n = new Vector2D(0,b.getAcceleration().magnitude()*Math.sin(phi)*dt);
				b.setVelocity(b.getVelocity().add(n.scalarMult(a.getMass()/b.getMass())));
			}*/
			
			return true;
		}
		//System.out.println("Corner collision.");
				
		/*Vector2D L;
		if (env.nextLoc(b,dt).getJ() > env.nextLoc(a,dt).getJ())
			L = env.nextLoc(b,dt).subtract(b.getHeightVector());
		else
			L = env.nextLoc(b,dt).add(b.getHeightVector());
		//System.out.println("L1 = " + L);
		if (env.getLoc(b).getI() > env.nextLoc(a,dt).getI())
			L = L.subtract(b.getWidthVector());
		else
			L = L.add(b.getWidthVector());
		L = L.subtract(env.nextLoc(a,dt));		//box b in local coordinates of circle a
		Vector2D dL = L.scalarDiv(L.magnitude()).scalarMult(a.getRadiusVector().magnitude()).subtract(L);*/
		//env.moveObject(a,dL.scalarMult(-1));
		//env.moveObject(b,dL.scalarMult(1));
		
		/*a.getVelocity().setJ(-a.getVelocity().getJ());
		a.getVelocity().setI(-a.getVelocity().getI());
		b.getVelocity().setJ(-b.getVelocity().getJ());
		b.getVelocity().setI(-b.getVelocity().getI());*/
		/*if (!a.bStatic())
		{
			w = new Vector2D(sx*STEP_SIZE,sy*STEP_SIZE);
			while (u < MAX_ITR &&
					a.collideAABB(b,env.getLoc(b).getI()-env.getLoc(a).getI(),env.getLoc(b).getJ()-env.getLoc(a).getJ()))
			{
				env.moveObject(a,w);
				u++;
			}
		} else
		{
			w = new Vector2D(-sx*STEP_SIZE,-sy*STEP_SIZE);
			while (u < MAX_ITR &&
					a.collideAABB(b,env.getLoc(b).getI()-env.getLoc(a).getI(),env.getLoc(b).getJ()-env.getLoc(a).getJ()))
			{
				env.moveObject(b,w);
				u++;
			}
		}*/
		
		if (!b.bStatic() && !a.bStatic())
		{
			b.getVelocity().setJ( (((b.getVelocity().getJ()*(b.getMass()-(E*a.getMass())))/(a.getMass()+b.getMass())) +
			((a.getVelocity().getJ()*(a.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
		
			b.getVelocity().setI( (((b.getVelocity().getI()*(b.getMass()-(E*a.getMass())))/(a.getMass()+b.getMass())) +
			((a.getVelocity().getI()*(a.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
			
			a.getVelocity().setJ( (((a.getVelocity().getJ()*(a.getMass()-(E*b.getMass())))/(a.getMass()+b.getMass())) +
			((by*(b.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
		
			a.getVelocity().setI( (((a.getVelocity().getI()*(a.getMass()-(E*b.getMass())))/(a.getMass()+b.getMass())) +
			((bx*(b.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
		}
		
		Vector2D w = new Vector2D(sx*STEP_SIZE,sy*STEP_SIZE);
		if (!a.bStatic())
			while (u < MAX_ITR && a.collideAABB(b,env.getLoc(b),env.getLoc(a)))
			{
				env.moveObject(a,w);
				u++;
			}
		else
			while (u < MAX_ITR && a.collideAABB(b,env.getLoc(b),env.getLoc(a)))
			{
				env.moveObject(b,w.scalarMult(-1));
				u++;
			}
		
		//normal forces
		Vector2D n = new Vector2D(env.getLoc(b).getI()-env.getLoc(a).getI(),env.getLoc(b).getJ()-env.getLoc(a).getJ());
		Vector2D vDir = n.scalarDiv(n.magnitude());
		n = n.scalarDiv(n.magnitude());
		n = n.scalarMult(b.getAcceleration().magnitude()).scalarMult(dt*Math.sin(b.getAcceleration().angle(n)));
		//System.out.println("N = " + n);
		if (!b.bStationary())
		{
			if (a.bStationary())
			{
				b.getVelocity().setAngle(vDir.getAngle());
				b.setVelocity(b.getVelocity().scalarMult(-E));
			}
			//normal force of a on b
			b.setVelocity(b.getVelocity().add(n));
			//System.out.println("Applying normal force of a on b.");
		}
		
		n = new Vector2D(env.getLoc(b).getI()-env.getLoc(a).getI(),env.getLoc(b).getJ()-env.getLoc(a).getJ());
		n = n.scalarDiv(n.magnitude());
		n = n.scalarMult(a.getAcceleration().magnitude()).scalarMult(dt*Math.sin(a.getAcceleration().angle(n)));
		
		if (!a.bStationary())
		{
			//normal force of b on a
			a.setVelocity(a.getVelocity().add(n));
			if (b.bStationary())
			{
				a.getVelocity().setAngle(vDir.getAngle());
				a.setVelocity(a.getVelocity().scalarMult(E));
			}
		}
		
		//normal force
		/*Vector2D n = new Vector2D(sx*a.getAcceleration().getI()*dt,sy*a.getAcceleration().getJ()*dt);
		a.setVelocity(a.getVelocity().add(n));	
		
		n = new Vector2D(-sx*a.getAcceleration().getI()*dt,-sy*a.getAcceleration().getJ()*dt);
		b.setVelocity(b.getVelocity().add(n));*/
		
		return true;	
	}
	
	private Vector2D CAFromSide(double xf, double yf, double x0, double y0, double bw, double bh)
	{	//returns location vector of circle relative to box
		boolean bx = AASideOverlap(x0,x0,-bw,bw), by = AASideOverlap(y0,y0,-bh,bh);
		if (bx && !by)	//if old center overlaps on x axis, return top/bottom
			return new Vector2D(0,y0);
		if (by && !bx)	//if overlap on y axis, return side collision
			return new Vector2D(x0,0);
		if (bx && by)	//if it is contained in the region
		{
			if (Math.abs(x0-bw) > Math.abs(y0-bh))		//check if it is closer to side or top/bottom
				return new Vector2D(0,y0);
			else
				return new Vector2D(x0,0);
		}
		//else came from a corner...
		if (AASideOverlap(xf,xf,-bw,bw))	//crossed to overlap on x
			return new Vector2D(0,yf);
		if (AASideOverlap(yf,yf,-bh,bh))	///...or on y
			return new Vector2D(xf,0);
		return new Vector2D(xf,yf);			//else intersected with corner first
	}
	
	public boolean collisionCO(PointMass a, PointMass b)
	{
		return false;
	}
	
	//Collision between two AABB's
	public boolean collisionAA(AABB a, AABB b)
	{
		//check for collision
		if (!a.collideAABB(b,env.nextLoc(b,dt).getI()-env.nextLoc(a,dt).getI(),env.nextLoc(a,dt).getJ()-env.nextLoc(b,dt).getJ()))
			return false;
		//respond to collision
		Vector2D v = AAFromSide(a,b);		//which side did 'a' come from?
		
		lastV = v;
		
		if (v == null)
			return false;
		
		//wake up if necessary
		/*if (a.getKE()/a.getMass() >= WAKE_KE && b.bSleeping())
		{
			b.wake();
		}*/
		
		//if from side
		if (v.getI() != 0 && v.getJ() == 0)
		{
			//System.out.println("Side collision...");				
			int s;
			if (env.getLoc(a).getI() < env.getLoc(b).getI())
					s = -1;
				else
					s = 1;
					
			int u = 0;
			/*double step,steps;
			if (a.getVelocity().getI()*dt > a.getWidth())
			{
				steps = a.getVelocity().getI()*dt/a.getWidth();
				step = a.getWidth();
			}
			else
			{
				steps = MAX_ITR;
				step = STEP_SIZE;
			}*/
			Vector2D w = new Vector2D(s*STEP_SIZE,0);
			while (u < MAX_ITR &&
					a.collideAABB(b,env.nextLoc(b,dt).getI()-env.nextLoc(a,dt).getI(),env.nextLoc(b,dt).getJ()-env.nextLoc(a,dt).getJ()))
			{
				env.moveObject(a,w);
				u++;
				//if (u == MAX_ITR)
				//	System.out.println("Could not fully resolve collision on x...");
			}
			//perfectly elastic collision, momentum conserved, energy conserved (Palmer, 
			if (!b.bStatic())		//don't move b if it is static
			{
				double vxa = a.getVelocity().getI();	//temporary initial values
				
				a.getVelocity().setI( (((a.getVelocity().getI()*(a.getMass()-(E_X*b.getMass())))/(a.getMass()+b.getMass())) +
									((b.getVelocity().getI()*(b.getMass()*(1+E_X)))/(a.getMass()+b.getMass()))));
				
				b.getVelocity().setI( (((b.getVelocity().getI()*(b.getMass()-(E_X*a.getMass())))/(a.getMass()+b.getMass())) +
					((vxa*(a.getMass()*(1+E_X)))/(a.getMass()+b.getMass()))));
				
				//b.getVelocity().setJ(E*b.getVelocity().getJ());
				
				//normal force of a on b
				Vector2D n = new Vector2D(-b.getAcceleration().getJ()*dt,0);
				b.setVelocity(b.getVelocity().add(n));
			}
			else		//static object collision special case
			{
				a.getVelocity().setI(s*E_X*Math.abs(a.getVelocity().getI()));
				//normal force
				Vector2D n = new Vector2D(-a.getAcceleration().getI()*dt,0);
				a.setVelocity(a.getVelocity().add(n));
			}
			
			//a.getVelocity().setJ(E*a.getVelocity().getJ());	
			
			return true;
		}
		//if from top/bottom
		if (v.getJ() != 0 && v.getI() == 0)
		{				
			//System.out.println("Top/bottom collision...");
			int s,sx;
				if (env.getLoc(a).getJ() < env.getLoc(b).getJ())
					s = -1;
				else
					s = 1;	
				if (env.getLoc(a).getI() < env.getLoc(b).getI())
					sx = -1;
				else
					sx = 1;	
			int u = 0;
			/*double step,steps;
			if (a.getVelocity().getJ()*dt > a.getHeight())
			{
				steps = a.getVelocity().getJ()*dt/a.getHeight();
				step = a.getHeight();
			}
			else
			{
				steps = MAX_ITR;
				step = STEP_SIZE;
			}*/
			
			Vector2D w = new Vector2D(0,s*STEP_SIZE);
			while (u < MAX_ITR &&
					a.collideAABB(b,env.nextLoc(b,dt).getI()-env.nextLoc(a,dt).getI()-sx*0,env.nextLoc(b,dt).getJ()-env.nextLoc(a,dt).getJ()-s*0))
			{
				env.moveObject(a,w);
				u++;
				//if (u == MAX_ITR)
					//System.out.println("Could not resolve collision on y...");
			}
					
			if (!b.bStatic())
			{
				double vya = a.getVelocity().getJ();	//temporary vya initial value
				
				a.getVelocity().setJ( (((a.getVelocity().getJ()*(a.getMass()-(E*b.getMass())))/(a.getMass()+b.getMass())) +
									((b.getVelocity().getJ()*(b.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
				
				b.getVelocity().setJ( (((b.getVelocity().getJ()*(b.getMass()-(E*a.getMass())))/(a.getMass()+b.getMass())) +
					((vya*(a.getMass()*(1+E)))/(a.getMass()+b.getMass()))));
				
				b.getVelocity().setI(E*b.getVelocity().getI());
				
				//normal force of a on b
				Vector2D n = new Vector2D(0,-b.getAcceleration().getJ()*dt);
				if (s>0)
					b.setVelocity(b.getVelocity().add(n));
				
			} else
			{
				a.getVelocity().setJ(-E*a.getVelocity().getJ());
				//normal force of b on a
				//System.out.println("Velocity before normal force: " + a.getVelocity());
				Vector2D n = new Vector2D(0,-a.getAcceleration().getJ()*dt);
				if (s<0)
					a.setVelocity(a.getVelocity().add(n));	
				//System.out.println("	Velocity before after force: " + a.getVelocity());
			}
			
			a.getVelocity().setI(E*a.getVelocity().getI());
			
			return true;
		}
		
		//otherwise just move the thing until it's out
		
		//budge in y direction		
		int s, u = 0;
		if (v.getJ() < 0)
			s = -1;
		else
			s = 1;
		
		Vector2D w = new Vector2D(0,s*STEP_SIZE);
		while (u < MAX_ITR &&
			a.collideAABB(b,env.nextLoc(b,dt).getI()-env.nextLoc(a,dt).getI(),env.nextLoc(b,dt).getJ()-env.nextLoc(a,dt).getJ()))
			{
			env.moveObject(a,w);
			u++;
		}
		
		//budge in x direction
		u = 0;
		if (v.getI() < 0)
			s = -1;
		else
			s = 1;
		
		w = new Vector2D(s*STEP_SIZE,0);
		while (u < MAX_ITR &&
			a.collideAABB(b,env.nextLoc(b,dt).getI()-env.nextLoc(a,dt).getI(),env.nextLoc(b,dt).getJ()-env.nextLoc(a,dt).getJ()))
			{
			env.moveObject(a,w);
			u++;
		}
		
		//System.out.println("AABB/AABB collision error. v = " + v);
		return true;
	}
	
	private Vector2D AAFromSide(AABB a, AABB b)		//returns vector pointing at direction of edge of collision
	{
		//calculate local coordinates of a (b at origin) and get object dimensions
		double xa = env.getLoc(a).getI()-env.getLoc(b).getI(),	//getpositions 
			ya = env.getLoc(a).getJ()-env.getLoc(b).getJ(), 	//in coordinates local to b
			wa = a.getWidth(), wb = b.getWidth(), ha = a.getHeight(), hb = b.getHeight();
			//xa = xa-((a.getVelocity().getI()+(a.getAcceleration().getI()*dt))*dt);
			//ya = ya-((a.getVelocity().getJ()+(a.getAcceleration().getJ()*dt))*dt);	//old positions
			//xa = a.oldLoc().getI()-env.getLoc(b).getI();
			//ya = a.oldLoc().getJ()-env.getLoc(b).getJ();
		boolean bX = AASideOverlap(xa,xa,-wb,wb);
		boolean bY = AASideOverlap(ya,ya,-hb,hb);
		if (bX && !bY)
		{
			//System.out.println("AASideOverlap:: top/bottom collision.");
			if (ya == 0)
				System.out.println("Error::AAFromSide: ya = 0.");
			return new Vector2D(0,ya);
		} if (bY && !bX)
		{
			//System.out.println("AASideOverlap:: side collision.");
			if (xa == 0)
				System.out.println("Error::AAFromSide: xa = 0.");
			return new Vector2D(xa,0);
		}
		
		//System.out.println("AABB came from a corner.");

		int sx, sy;
		if (xa < 0)
			sx = -1;
		else
			sx = 1;
		if (ya < 0)
			sy = -1;
		else
			sy = 1;
			
		if (a.getVelocity().getI()*sx >= 0)
			return new Vector2D(0,ya);
		if (a.getVelocity().getJ()*sy >= 0)
			return new Vector2D(0,xa);
		
		double nxa = xa - sx*(wb + wa);
		double nya = ya - sy*(hb + ha);
		
		//System.out.println("	Conditional: " + ((nya)/a.getVelocity().getJ()) + " <= " + (nxa)/a.getVelocity().getI() + "?");
		
		//velocity components will never be zero(?)
		if (a.getVelocity().getI() == 0 || ((nya)/a.getVelocity().getJ()) <= ((nxa)/a.getVelocity().getI()))
		{
			//System.out.println("	Vertical collision; returning " + new Vector2D (xa,0));
			return new Vector2D (0,ya);
		}
		else
		{
			//System.out.println("	Horizontal collision; returning " + new Vector2D (0,ya));
			return new Vector2D(xa,0);
		}
	}
	
	public static boolean AASideOverlap(double a1, double a2, double b1, double b2)
	{	//check if two one-dimensional line segments overlap/intersect. pass in left to right
		//System.out.println(new Vector2D(a1,a2) + " " + new Vector2D(b1,b2) + ": overlap testing...");
		if (b1 > b2)
		{
			double t = b1;
			b1 = b2;
			b2 = t;
		}
		if (a1 < b1 && a2 < b1)		//easier to test to see if not overlapping
		{
			//System.out.println("Side overlap FALSE between: " + a1 + "," + a2 + " & " +b1+","+b2);
			return false;
		}
		if (a1 > b2 && a2 > b2)
		{
			//System.out.println("Side overlap FALSE between: " + a1 + "," + a2 + " & " +b1+","+b2);
			return false;
		}
		//System.out.println("Side overlap true between: " + a1 + "," + a2 + " & " +b1+","+b2);
		return true;
	}
	
	public boolean collisionAO(PointMass a, PointMass b)
	{
		return false;
	}
	
	public boolean collisionOO(PointMass a, PointMass b)
	{
		return false;
	}
	
	public static double pointLineSqDist(double ax, double ay, double bx, double by)
	{	//finds squared distance between a point and a line (local coordinates, origin is point)
		System.out.println("ax = " + ax + "; ay = " + ay + "; bx = " + bx + "; by = " + by);
		double dy = bx + ax;
		double dx = by + ay;
		double z = (ax*dx + ay*dy);
		z = z / (dx*dx + dy*dy);
		System.out.println("z = " + z);
		if (z < 0)
			z = 0;
		else if (z > 1)
			z = 1;
		System.out.println("pointLineSqDist actual dist: " + Math.sqrt((-ax + z*dx)*(-ax + z*dx) + (-ay + z*dy)*(-ay + z*dy)));
		return (-ax + z*dx)*(-ax + z*dx) + (-ay + z*dy)*(-ay + z*dy);
	}
	
	public static double pointLineSqDist(Vector2D A, Vector2D B, Vector2D C)
	{	//finds squared distance between a point and a line (location vectors)
		Vector2D AB = B.subtract(A);
		double x = C.subtract(A).dot(AB)/AB.dot(AB);
		if (x < 0)
			x = 0;
		if (x > 1)
			x = 1;
		Vector2D d = A.add(AB.scalarMult(x)).subtract(C);
		return d.getI()*d.getI() + d.getJ()*d.getJ();
	}
	
	public double getDT()
	{
		return dt;
	}
	
	public PhysicsEnv getEnv()
	{
		return env;
	}
	
	public static Vector2D rayTrace(Vector2D loc, Vector2D r, PhysicsEnv e)
	{
		if (r.squaredMag() != 1)
			r = r.scalarDiv(r.magnitude());
		
		ArrayList l = new ArrayList();
		ArrayList objs = e.getAllObjects();
		Vector2D[] t = new Vector2D[2];
		PointMass p;
		
		for (int i = 0; i < objs.size(); i++)
		{
			p = (PointMass)(objs.get(i));
			if (p.CLASS() == 1)
				t = rayTrace(loc,r,e,(BoundingCircle)p);
			else if (p.CLASS() == 2)
				t = rayTrace(loc,r,e,(AABB)p);
			if (t != null)
			{
				l.add(t[0]);
				l.add(t[1]);
			}
		}
		double min = 99999999;
		int index = 0;
		Vector2D v;
		for (int i = 0; i < l.size(); i++)
		{
			v = (Vector2D)(l.get(i));
			if (v != null && v.squaredMag() < min)
			{
				min = v.squaredMag();
				index = i;
			}
		}
		
		return (Vector2D)l.get(index);
	}
	
	public static Vector2D[] rayTrace(Vector2D loc, Vector2D r, PhysicsEnv e, BoundingCircle a)
	{
		Vector2D[] l = new Vector2D[2];
		Vector2D c = e.getLoc(a).subtract(loc);
		double t1 = r.dot(c);
		t1 = t1*t1;
		double t2 = t1 - c.dot(c) + a.getRadius()*a.getRadius();
		if (t2 < 0)
			return null;
		double sq = Math.sqrt(t2);
		double x1 = r.dot(c) + sq, x2 = r.dot(c) - sq;
		if (x1 >= 0)
			l[0] = r.scalarMult(x1);
		if (x2 >= 0)
			l[1] = r.scalarMult(x2);
		return l;
	}
	
	public static Vector2D[] rayTrace(Vector2D loc, Vector2D r, PhysicsEnv e, AABB a)
	{
		Vector2D[] l = new Vector2D[2];
		Vector2D c = e.getLoc(a).subtract(loc);
		if (r.getI() == 0 || r.getJ() == 0)
			return null;		//temporary escape
		double yP = (c.getJ() + a.getHeight())/r.getJ();
		double yN = (c.getJ() - a.getHeight())/r.getJ();
		double xP = (c.getI() + a.getWidth())/r.getI();
		double xN = (c.getI() - a.getWidth())/r.getI();
		System.out.println("yP = " + yP + "; yN = " + yN + "; xP = " + xP + "; xN = " + xN);
		if (yP >= 0 && AASideOverlap(yP,yP,xN,xP))
		{
			l[0] = r.scalarMult(yP);
		}
		if (yN >= 0 && AASideOverlap(yN,yN,xN,xP))
		{
			l[1] = r.scalarMult(yN);
		}
		if (xP >= 0 && AASideOverlap(xP,xP,yP,yN))
		{
			l[0] = r.scalarMult(xP);
		}
		if (xN >= 0 && AASideOverlap(xN,xN,yP,yN))
		{
			l[1] = r.scalarMult(xN);
		}
		System.out.println("l = " + l[0] + ", " + l[1]);
		return l;
	}
}