import java.util.*;

/*#############################
 *	Author: Mark Walker
 *	Class: SwimmerDemo.java
 *		A simple game that lets you move an object around
 *############################*/

public class SwimmerDemo implements GameEngine
{
	private PhysicsEngine2 phys;
	
	private int HEIGHT = 5, WIDTH = 5, MASS = 10, MBOX = 1000, GRAVITY = 500, V_EXPL = 2000, FORCE = 100, RADIUS = 20,
		ROWS = 5, COLS = 5, SPACE = 30, OFFSET = 200;
	public boolean bStatic, bBoxes;
	AABB theBox;
	
	public SwimmerDemo()
	{
		phys = new PhysicsEngine2();
		
		bStatic = false;
		bBoxes = true;
		
		BoundingCircle c;
		for (int i = 0; i < ROWS; i++)
		{
			for (int j = 0; j < COLS; j++)
			{
				c = new BoundingCircle(MASS,RADIUS,
					new Vector2D(j*SPACE+OFFSET,i*SPACE+OFFSET));
				getEnv().addObject(c);
				c.setAccel(new Vector2D(0,GRAVITY));
			}
		}
		
		AABB b = new AABB(1,900,200,new Vector2D(400,800));		//border boxes
		b.setStatic(true);
		AABB f = new AABB(1,900,200,new Vector2D(400,0));
		f.setStatic(true);
		AABB d = new AABB(1,200,900,new Vector2D(0,400));
		d.setStatic(true);
		AABB e = new AABB(1,200,900,new Vector2D(800,400));
		e.setStatic(true);
		getEnv().addObject(b);
		getEnv().addObject(f);
		getEnv().addObject(d);
		getEnv().addObject(e);
		
		theBox = new AABB(MBOX,100,50,new Vector2D(600,600));
		getEnv().addObject(theBox);
		
		explosion(0,0);
	}
	
	public void changeStaticMode()
	{
		System.out.println("Static mode changed.");
		bStatic = !bStatic;
	}
	
	public boolean bBoxes()
	{
		return bBoxes;
	}
	
	public void addObject(int x, int y)
	{
	/*	if (bBoxes)
			addBox(x,y);
		else
			addCircle(x,y);*/
	}
	
	public void toggleBoxes()
	{
		//System.out.println("Object type toggled.");
		//bBoxes = !bBoxes;
	}
	
	public void addBox(int x, int y)
	{
		//System.out.println("Adding box at location (" + x + ", " + y + ")");
		AABB box = new AABB(MASS,WIDTH,HEIGHT,new Vector2D(x,y));
		getEnv().addObject(box);
		box.setAccel(new Vector2D(0,GRAVITY));	
		if (bStatic)
			box.setStatic(true);	
	}
	
	public void addCircle(int x, int y)
	{
		//System.out.println("Adding circle at location (" + x + ", " + y + ")");
		BoundingCircle circle = new BoundingCircle(MASS,RADIUS,new Vector2D(x,y));
		getEnv().addObject(circle);
		circle.setAccel(new Vector2D(0,GRAVITY));	
		if (bStatic)
			circle.setStatic(true);	
	}
	
	public void explosion(int x, int y)
	{
		//System.out.println("EXPLOSION!");
		ArrayList l = getEnv().getAllObjects();
		AABB box;
		BoundingCircle circle;
		for (int i = 0; i < l.size(); i++)
		{
			if (((PointMass)l.get(i)).CLASS() == 2)
			{
				box = (AABB)l.get(i);
				if (!box.bStatic() || box.bSleeping())
				{
					box.setVelocity(new Vector2D(V_EXPL*-Math.sin(getEnv().getLoc(box).getI()/2),
						V_EXPL*-Math.cos(getEnv().getLoc(box).getJ()/2)));
					box.wake();
				}
			} else if (((PointMass)l.get(i)).CLASS() == 1)
			{
				circle = (BoundingCircle)l.get(i);
				if (!circle.bStatic() || circle.bSleeping())
				{
					circle.setVelocity(new Vector2D(V_EXPL*-Math.sin(getEnv().getLoc(circle).getI()/2),
					//-V_EXPL));
					V_EXPL*-Math.cos(getEnv().getLoc(circle).getJ()/2)));
					circle.wake();
				}
			}
		}
	}
	
	public void hitBox(Vector2D d)
	{
		theBox.applyForce(d.scalarMult(FORCE));
	}
	
	public void act()
	{
		phys.act();
		if (theBox.bSleeping())
			theBox.wake();
	}
	
	public PhysicsEnv getEnv()
	{
		return phys.getEnv();
	}
}